@@ -35,7 +35,7 @@ Boston, MA 02111-1307, USA.
    would necessitate including windows.h in the files that used it.
    This is much easier.  */
 void
-sleep(int seconds)
+sleep(unsigned long seconds)
 {
   Sleep (seconds * 1000);
 }
@@ -58,7 +58,7 @@ getppid(void)
   char *ppid;
   DWORD result;
 
-  ppid = getenv ("__PARENT_PROCESS_ID");
+  ppid = getenv ("EM_PARENT_PROCESS_ID");
   if (!ppid) 
     {
       printf("no pid.\n");
