@@ -1,32 +1,33 @@
-;;; env.el --- functions to manipulate environment variables.
+;;; env.el --- functions to manipulate environment variables
 
-;;; Copyright 1991, 1994 Free Software Foundation, Inc.
+;; Copyright (C) 1991, 1994, 2000, 2001 Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
 ;; Keywords: processes, unix
 
-;;; This file is part of GNU Emacs.
+;; This file is part of GNU Emacs.
 
-;;; GNU Emacs is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
 
-;;; GNU Emacs is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
 
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to
-;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
-;; UNIX processes inherit a list of name-to-string associations from
-;; their parents called their `environment'; these are commonly used
-;; to control program options.  This package permits you to set
-;; environment variables to be passed to any sub-process run under Emacs.
+;; UNIX processes inherit a list of name-to-string associations from their
+;; parents called their `environment'; these are commonly used to control
+;; program options.  This package permits you to set environment variables
+;; to be passed to any sub-process run under Emacs.
 
 ;;; Code:
 
@@ -48,7 +49,6 @@ If it is also not t, RET does not exit if it does non-null completion."
 ;; History list for VALUE argument to setenv.
 (defvar setenv-history nil)
 
-;;;###autoload
 (defun setenv (variable &optional value unset)
   "Set the value of the environment variable named VARIABLE to VALUE.
 VARIABLE should be a string.  VALUE is optional; if not provided or is
@@ -63,28 +63,13 @@ This function works by modifying `process-environment'."
    (if current-prefix-arg
        (list (read-envvar-name "Clear environment variable: " 'exact) nil t)
      (let* ((var (read-envvar-name "Set environment variable: " nil))
-	    (oldval (getenv var))
-	    newval
-	    oldhist)
-       ;; Don't put the current value on the history
-       ;; if it is already there.
-       (if (equal oldval (car setenv-history))
-	   (setq oldval nil))
-       ;; Now if OLDVAL is non-nil, we should add it to the history.
-       (if oldval
-	   (setq setenv-history (cons oldval setenv-history)))
-       (setq oldhist setenv-history)
-       (setq newval (read-from-minibuffer (format "Set %s to value: " var)
-					  nil nil nil 'setenv-history))
-       ;; If we added the current value to the history, remove it.
-       ;; Note that read-from-minibuffer may have added the new value.
-       ;; Don't remove that!
-       (if oldval
-	   (if (eq oldhist setenv-history)
-	       (setq setenv-history (cdr setenv-history))
-	     (setcdr setenv-history (cdr (cdr setenv-history)))))
+	    (value (getenv var)))
+       (when value
+	 (push value setenv-history))
        ;; Here finally we specify the args to give call setenv with.
-       (list var newval))))
+       (list var (read-from-minibuffer (format "Set %s to value: " var)
+				       nil nil nil 'setenv-history
+				       value)))))
   (if unset (setq value nil))
   (if (string-match "=" variable)
       (error "Environment variable name `%s' contains `='" variable)
@@ -92,6 +77,8 @@ This function works by modifying `process-environment'."
 	  (case-fold-search nil)
 	  (scan process-environment)
 	  found)
+      (if (string-equal "TZ" variable)
+	  (set-time-zone-rule value))
       (while scan
 	(cond ((string-match pattern (car scan))
 	       (setq found t)
@@ -106,6 +93,19 @@ This function works by modifying `process-environment'."
 		    (cons (concat variable "=" value)
 			  process-environment)))))))
 
+(defun getenv (variable)
+  "Get the value of environment variable VARIABLE.
+VARIABLE should be a string.  Value is nil if VARIABLE is undefined in
+the environment.  Otherwise, value is a string.
+
+This function consults the variable `process-environment'
+for its value."
+  (interactive (list (read-envvar-name "Get environment variable: " t)))
+  (let ((value (getenv-internal variable)))
+    (when (interactive-p)
+      (message "%s" (if value value "Not set")))
+    value))
+
 (provide 'env)
 
 ;;; env.el ends here
