@@ -620,6 +620,9 @@ extern void coding_restore_composition P_ ((struct coding_system *,
 					    Lisp_Object));
 extern int code_convert_region P_ ((int, int, int, int, struct coding_system *,
 				    int, int));
+extern Lisp_Object run_pre_post_conversion_on_str P_ ((Lisp_Object,
+						       struct coding_system *,
+						       int));
 extern int decoding_buffer_size P_ ((struct coding_system *, int));
 extern int encoding_buffer_size P_ ((struct coding_system *, int));
 extern void detect_coding P_ ((struct coding_system *, unsigned char *, int));
