@@ -15,25 +15,18 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 
 /* The following line tells the configuration script what sort of 
    operating system this machine is likely to run.
    USUAL-OPSYS="bsd4-2"  */
 
-/* The following three symbols give information on
- the size of various data types.  */
+/* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
+   is the most significant byte.  */
 
-#define SHORTBITS 16		/* Number of bits in a short */
-
-#define INTBITS 32		/* Number of bits in an int */
-
-#define LONGBITS 32		/* Number of bits in a long */
-
-/* 68000 has lowest-numbered byte as most significant */
-
-#define BIG_ENDIAN
+#define WORDS_BIG_ENDIAN
 
 /* XINT must explicitly sign-extend */
 
