@@ -39,3 +39,12 @@ extern int executing_macro_iterations;
    so after an error, it describes the innermost interrupted macro.  */
 
 extern Lisp_Object executing_macro;
+
+/* Declare that all chars stored so far in the kbd macro being defined
+ really belong to it.  This is done in between editor commands.  */
+
+extern void finalize_kbd_macro_chars P_ ((void));
+
+/* Store a character into kbd macro being defined */
+
+extern void store_kbd_macro_char P_ ((Lisp_Object));
