@@ -19,10 +19,11 @@ the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-/*
- * Structure records pertinent information about open channels.
- * There is one channel associated with each process.
- */
+/* This structure records information about a subprocess
+   or network connection.
+
+   Every field in this structure except for the first two
+   must be a Lisp_Object, for GC's sake.  */
 
 struct Lisp_Process
   {
@@ -79,12 +80,22 @@ struct Lisp_Process
     Lisp_Object decode_coding_system;
     /* Working buffer for decoding.  */
     Lisp_Object decoding_buf;
+    /* Size of carryover in decoding.  */
+    Lisp_Object decoding_carryover;
     /* Coding-system for encoding the output to this process.  */
     Lisp_Object encode_coding_system;
     /* Working buffer for encoding.  */
     Lisp_Object encoding_buf;
+    /* Size of carryover in encoding.  */
+    Lisp_Object encoding_carryover;
+    /* Flag to set coding-system of the process buffer from the
+       coding_system used to decode process output.  */
+    Lisp_Object inherit_coding_system_flag;
 };
 
+/* Every field in the preceding structure except for the first two
+   must be a Lisp_Object, for GC's sake.  */
+
 #define ChannelMask(n) (1<<(n))
 
 /* Indexed by descriptor, gives the process (if any) for that descriptor.  */
