@@ -1,5 +1,5 @@
-/* Definitions file for GNU Emacs running on Sony's NEWS-OS 5.0.1
-   Copyright (C) 1992 Free Software Foundation, Inc.
+/* Definitions file for GNU Emacs running on Sony's NEWS-OS 5.0.2
+   Copyright (C) 1992, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,11 +15,12 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* Use the SysVr4 file for at least base configuration. */
 
-#include "s-usg5-4.h"
+#include "usg5-4.h"
 
 #define NEWSOS5
 
@@ -27,11 +28,22 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #undef START_FILES
 #undef LIB_STANDARD
 
-/* Disable use of "unexelf.c" and shared libraries,
-   because NEWS-OS on MIPS doesn't support ELF. */
+#undef LIBS_SYSTEM
+#define LIBS_SYSTEM -lsocket -lnsl -lgen
+
+/* Disable use of "unexelf.c" and shared libraries, because
+   "unexelf.c" doesn't work correctly on NEWS-OS.  "unexmips.c" does
+   work correctly if the program is linked statically without ELF. */
 #undef UNEXEC
 #undef USG_SHARED_LIBRARIES
 
+/* Use `ld' directly rather than ordinary link, because ordinary link
+   can't produce a non-ELF executable.  */
+#undef ORDINARY_LINK
+#define LINKER /usr/lib/cmplrs/cc/ld
+#define START_FILES pre-crt0.o /usr/ccs/lib/crt1.o
+#define LIB_STANDARD -lc /usr/ccs/lib/crtn.o /usr/ccs/lib/values-Xt.o
+
 #ifndef HAVE_SOCKETS
 #define HAVE_SOCKETS
 #endif
