@@ -2,6 +2,10 @@
 
 #include "sol2-4.h"
 
+#if 0 /* Klaus Zeitler <kzeitler@lucent.com> says SIGIO still fails.  */
+#undef BROKEN_SIGIO
+#endif
+
 /* -lgen is needed for the regex and regcmp functions
    which are used by Motif.  In the future we can try changing
    regex.c to provide them in Emacs, but this is safer for now.  */
