@@ -23,27 +23,28 @@
 #define LD_SWITCH_SYSTEM `./prefix-args -Xlinker LD_SWITCH_X_SITE_AUX`
 #endif /* GCC */
 
-/* Compile in non-ansi fashion to work around bugs in system header files.  */
+#undef LIBS_SYSTEM
+#define LIBS_SYSTEM -lsocket -lnsl -lkstat
+
+/* Prefer kstat over kvm in getloadavg.c, kstat doesn't require root.
+   ghazi@caip.rutgers.edu, 7/21/97. */
+#define HAVE_LIBKSTAT
+
+/* eggert thinks all versions of SunPro C allowed this.  */
 #ifndef __GNUC__
-#define C_SWITCH_SYSTEM -Xs
-#else /* GCC */
-#define C_SWITCH_SYSTEM -traditional
-#endif /* GCC */
-#define const
-
-/* Karl Berry writes:
-If you have the misfortune to be running Solaris 2.1, you may have
-noticed that the access system call does not check the readonlyness of
-the filesystem the path refers to.  This is a bug, according to
-access(2), but in the meantime, some of us need the right behavior.  */
-
-/* Well, we released Emacs with this change, and fixed a typo, but
-   people keep saying that it doesn't work, and that the patch is easy
-   to install.  Patch number is 100947-02.  */
-#undef SOLARIS_BROKEN_ACCESS
-
-/* We can't use libX11 from X11R6 with ralloc.c, because libX11
-   includes libthread.so and that calls sbrk directly.  */
-#if defined (HAVE_X11) && defined (HAVE_X11R6)
-#undef REL_ALLOC
+#define C_DEBUG_SWITCH -g -O
 #endif
+
+/* inoue@ainet.or.jp says Solaris has a bug related to X11R6-style
+   XIM support.  */
+
+#define INHIBIT_X11R6_XIM
+
+/* Must use the system's termcap, if we use any termcap.
+   It does special things.  */
+
+#ifndef TERMINFO
+#define LIBS_TERMCAP -ltermcap
+#endif
+
+#define USE_MMAP_FOR_BUFFERS 1
